from whoosh.analysis import Tokenizer, Token
import MeCab

class MeCabTokenizer(Tokenizer):
    def __init__(self, conf=''):
        self.conf = conf
        self.tagger = MeCab.Tagger(conf)

    def __getstate__(self):
        return dict([(k, self.__dict__[k]) for k in self.__dict__
                     if k != "tagger"])

    def __setstate__(self, state):
        self.__dict__.update(state)
        self.tagger = MeCab.Tagger(conf)

    def __call__(self, value, positions=False, chars=False,
                 keeporiginal=False, removestops=True,
                 start_pos=0, start_char=0,
                 tokenize=True, mode='', **kwargs):
        assert isinstance(value, unicode), "%r is not unicode" % value
        t = Token(positions, chars, removestops=removestops, mode=mode)
        if not tokenize:
            t.original = t.text = value
            t.boost = 1.0
            if positions:
                t.pos = start_pos
            if chars:
                t.startchar = start_char
                t.endchar = start_char + len(value)
            yield t
        else:
            pos = start_pos
            offset = start_char
	    byte_offset = 0
            # TODO: support other encodings
	    byte = value.encode('utf-8')
            m = self.tagger.parseToNode(byte)
            while m:
	        if len(m.surface) == 0:
		    m = m.next
		    continue
                t.text = m.surface.decode('utf-8')
                t.feature = m.feature
                # TODO: use base form.
                t.boost = 1.0
                if keeporiginal:
                    t.original = t.text
                t.stopped = False
                if positions:
                    t.pos = pos
                    pos += 1
                if chars:
		    s = byte_offset + m.rlength - m.length
                    e = s + m.length
                    t.startchar = offset + len(byte[byte_offset:s].decode('utf-8'))
                    t.endchar = t.startchar + len(byte[s:e].decode('utf-8'))
                    offset = t.endchar
                    byte_offset = e
                m = m.next
                yield t



# coding:utf-8
from whoosh.analysis import Tokenizer, Token
import igo.Tagger


class IgoTokenizer(Tokenizer):
    def __init__(self, tagger=None, **tagger_initparam):
        if tagger:
            self.tagger = tagger
        else:
            self.tagger = igo.Tagger.Tagger(**tagger_initparam)
        self.tagger_initparam = tagger_initparam

    def __getstate__(self):
        if not self.tagger_initparam:
            return self.__dict__
        return dict([(k, self.__dict__[k]) for k in self.__dict__
                     if k != "tagger"])

    def __setstate__(self, state):
        self.__dict__.update(state)
        if not hasattr(self, 'tagger'):
            self.tagger = igo.Tagger.Tagger(**self.tagger_initparam)

    def __call__(self, value, positions=False, chars=False,
                 keeporiginal=False, removestops=True,
                 start_pos=0, start_char=0,
                 tokenize=True, mode='', **kwargs):
        assert isinstance(value, unicode), "%r is not unicode" % value
        t = Token(positions, chars, removestops=removestops, mode=mode)
        if not tokenize:
            t.original = t.text = value
            t.boost = 1.0
            if positions:
                t.pos = start_pos
            if chars:
                t.startchar = start_char
                t.endchar = start_char + len(value)
            yield t
        else:
            pos = start_pos
            for m in self.tagger.parse(value):
                t.text = m.surface
                t.feature = m.feature
                # TODO: use base form.
                t.boost = 1.0
                if keeporiginal:
                    t.original = t.text
                t.stopped = False
                if positions:
                    t.pos = pos
                    pos += 1
                if chars:
                    t.startchar = start_char + m.start
                    t.endchar = t.startchar + len(m.surface)
                yield t



import re
from whoosh.analysis import Filter


class FeatureFilter(Filter):
    def __init__(self, ignores):
        self.ignores = [re.compile(x, re.U) for x in ignores]

    @staticmethod
    def find_match(f, ignores):
        for pat in ignores:
            if pat.match(f):
                return True
        return False

    def __call__(self, tokens):
        ignores = self.ignores
        find_match = FeatureFilter.find_match
        pos = None
        for t in tokens:
            if find_match(t.feature, ignores):
                if not t.removestops:
                    t.stopped = True
                    yield t
            else:
                if t.positions:
                    if pos is None:
                        pos = t.pos
                    else:
                        pos += 1
                        t.pos = pos
                t.stopped = False
                yield t



from whoosh.analysis import Tokenizer, Token

class TinySegmenterTokenizer(Tokenizer):
    def __init__(self, segmenter, strip=True):
        self.segmenter = segmenter
        self.strip = strip

    def __call__(self, value, positions=False, chars=False,
                 keeporiginal=False, removestops=True,
                 start_pos=0, start_char=0,
                 tokenize=True, mode='', **kwargs):
        assert isinstance(value, unicode), "%r is not unicode" % value
        t = Token(positions, chars, removestops=removestops, mode=mode)
        if not tokenize:
            t.original = t.text = value
            t.boost = 1.0
            if positions:
                t.pos = start_pos
            if chars:
                t.startchar = start_char
                t.endchar = start_char + len(value)
            yield t
        else:
            if self.strip:
                def strip(s):
                    return s.strip()
            else:
                def strip(s):
                    return s

            pos = start_pos
            startchar = start_char
            for s, l in [(strip(s), len(s)) for s in self.segmenter.tokenize(value)]:
                t.text = s
                t.boost = 1.0
                if keeporiginal:
                    t.original = t.text
                t.stopped = False
                if positions:
                    t.pos = pos
                    pos += 1
                if chars:
                    t.startchar = startchar
                    startchar += l
                    t.endchar = startchar
                yield t



#!/usr/bin/env python
# encoding: utf-8

from setuptools import setup


setup(
    name='whoosh-igo',
    version='0.6',
    description='tokenizers for Whoosh designed for Japanese language',
    long_description = open('README').read() + "\n\n" + open('CHANGES').read(),
    author='Hideaki Takahashi',
    author_email='mymelo@gmail.com',
    url='https://launchpad.net/igotokenizer/',
    classifiers=['Development Status :: 4 - Beta',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: Apache Software License',
                 'Natural Language :: Japanese',
                 'Operating System :: OS Independent',
                 'Operating System :: Microsoft :: Windows',
                 'Operating System :: POSIX :: Linux',
                 'Programming Language :: Python :: 2.5',
                 'Programming Language :: Python :: 2.6',
                 'Programming Language :: Python :: 2.7',
                 'Topic :: Scientific/Engineering :: Information Analysis',
                 'Topic :: Software Development :: Libraries :: Python Modules',
                 'Topic :: Text Processing :: Linguistic',
                 ],
    keywords=['japanese', 'tokenizer',],
    license='Apache License, Version 2.0',
    packages=['WhooshJapaneseTokenizer'],
    )
